* Before this file can be executed:
*  - first solve the main model for all desired parameter specifications
*  - then copy all gdx results files into the respective project folder

* Merge all results into one gdx file
$call gdxmerge eta_-0.*alpha_0.*_MPen*.gdx eta_-0.*alpha_0.*_MPcap*.gdx

set
T                Hour t in given year    /1*8760/
I                /Wind_on, PV, Coal, CCGT, OCGT_Oil/
IR(I)            /Wind_on, PV/
set_eta          /'-0.01', '-0.05', '-0.1'/
help             /alpha, eta, a, m, p0, prtp, pflat, mp_levy, MP_var_en, 'Total gen', 'RES share', Capacities, Generation, Welfare/
CR_index         /RTP,RTP_neg_price,RTP_pos_price,FRP,FRP_neg_price,FRP_pos_price,total/
delta_index      /CR_RTP, CR_FRP, CR_SWITCH, CR_total/
merged_set_1
;

parameters
report
report_I
report_T
report_I_T
instr
eta
alpha
a
m
p0
prtp
pflat
mp_levy
total_gen
RES_share
OpCa
q
qrtp
qfrp
co2
fc
mc
pcarb

cost_sum                 Total costs of generation
q_sum                    Total generation
turnover                 Prices times quantities
CR_net                   Net consumer rent
CR_gross                 Gross consumer rent
Delta_instr              change of outcomes between premium on capacity and premium on energy
Delta_alpha              change of consumer rent between different RTP shares
euro_per_MWh             Diverse results measured in euro per MWh
market_value             Market value of fluctuating renewables
p_retail_rtp             Retail price of real time price consumers
p_retail_frp             Retail price of flat rate price consumers
p_retail_rtp_neg         Retail price of real time price consumers - in hours with negative prices
p_retail_frp_neg         Retail price of flat rate price consumers - in hours with negative prices
;

$gdxin merged.gdx
$load merged_set_1
$load report
$load report_T
$load report_I
$load report_I_T
$gdxin

$gdxin input_data.gdx
$load co2
$load fc
$load mc
$gdxin

* Set instr to 1 for MPcap and 2 for MPen
instr(merged_set_1) = 1+1$report(merged_set_1,'MP_var_en') ;

* Read in model results
eta(merged_set_1) = report(merged_set_1,'eta') ;
alpha(merged_set_1) = report(merged_set_1,'alpha') ;
a(merged_set_1,T) = report_T(merged_set_1,'a',T) ;
m(merged_set_1) = report(merged_set_1,'m') ;
p0(merged_set_1,T) = report_T(merged_set_1,'p0',T) ;
prtp(merged_set_1,T) = report_T(merged_set_1,'prtp',T) ;
pflat(merged_set_1) = report(merged_set_1,'pflat') ;
mp_levy(merged_set_1) = report(merged_set_1,'mp_levy') ;
total_gen(merged_set_1) = report(merged_set_1,'Total gen') ;
RES_share(merged_set_1) = report(merged_set_1,'RES share') ;
OpCa(merged_set_1,I) = report_I(merged_set_1,'Capacities',I) ;
q(merged_set_1,I,T) = report_I_T(merged_set_1,'Generation',I,T) ;

pcarb = 0 ;

* Calculate qrtp and qfrp:
qrtp(merged_set_1,T) = (prtp(merged_set_1,T)+mp_levy(merged_set_1)-p0(merged_set_1,T))/m(merged_set_1) ;
qfrp(merged_set_1,T) = (pflat(merged_set_1)+mp_levy(merged_set_1)-p0(merged_set_1,T))/m(merged_set_1) ;

* Generate auxiliary subset to exclude alpha 0.01
set subset_alpha_01 ;
subset_alpha_01(merged_set_1)$(alpha(merged_set_1)=0.01) = yes ;

* Generate subsets for instruments - only defined for 50% RES share
set subset_MPen(merged_set_1), subset_MPcap(merged_set_1) ;
subset_MPcap(merged_set_1)$((instr(merged_set_1)=1) and (RES_share(merged_set_1)>0.49) and (RES_share(merged_set_1)<0.51)) = yes ;
subset_MPen(merged_set_1)$((instr(merged_set_1)=2) and (RES_share(merged_set_1)>0.49) and (RES_share(merged_set_1)<0.51))= yes ;

* Make notation easier:
loop(merged_set_1,
         p_retail_rtp(merged_set_1,T) = prtp(merged_set_1,T)+mp_levy(merged_set_1) ;
         p_retail_frp(merged_set_1) = pflat(merged_set_1)+mp_levy(merged_set_1) ;
);

* Selection of hours with negative retail prices
loop(merged_set_1,
         p_retail_rtp_neg(merged_set_1,T) = p_retail_rtp(merged_set_1,T)$( p_retail_rtp(merged_set_1,T) < 0 ) ;
         p_retail_frp_neg(merged_set_1) = p_retail_frp(merged_set_1)$( p_retail_frp(merged_set_1) < 0 ) ;
);

* Calculation of net consumer surplus of RTP and FRP consumers
loop(merged_set_1,
         CR_net(merged_set_1,'RTP_pos_price') = alpha(merged_set_1) * sum(T$(not p_retail_rtp_neg(merged_set_1,T)), (sqr(p0(merged_set_1,T))-sqr(p_retail_rtp(merged_set_1,T)))/(-2*m(merged_set_1)) - qrtp(merged_set_1,T)*p_retail_rtp(merged_set_1,T) ) ;
         CR_net(merged_set_1,'RTP_neg_price') = alpha(merged_set_1) * sum(T$p_retail_rtp_neg(merged_set_1,T), (sqr(p0(merged_set_1,T)-p_retail_rtp(merged_set_1,T)))/(-2*m(merged_set_1)) ) ;
         CR_net(merged_set_1,'RTP') = CR_net(merged_set_1,'RTP_pos_price') + CR_net(merged_set_1,'RTP_neg_price');
         CR_gross(merged_set_1,'RTP_pos_price') = alpha(merged_set_1) * sum(T$(not p_retail_rtp_neg(merged_set_1,T)), (sqr(p0(merged_set_1,T))-sqr(p_retail_rtp(merged_set_1,T)))/(-2*m(merged_set_1)) ) ;
         CR_gross(merged_set_1,'RTP_neg_price') = alpha(merged_set_1) * sum(T$p_retail_rtp_neg(merged_set_1,T), (sqr(p0(merged_set_1,T))-sqr(p_retail_rtp(merged_set_1,T)))/(-2*m(merged_set_1)) ) ;
         CR_gross(merged_set_1,'RTP') = CR_gross(merged_set_1,'RTP_pos_price') + CR_gross(merged_set_1,'RTP_neg_price');

         CR_net(merged_set_1,'FRP_pos_price') = (1-alpha(merged_set_1)) * sum(T$(not p_retail_frp_neg(merged_set_1)), (sqr(p0(merged_set_1,T))-sqr(p_retail_frp(merged_set_1)))/(-2*m(merged_set_1)) - qfrp(merged_set_1,T)*p_retail_frp(merged_set_1) ) ;
         CR_net(merged_set_1,'FRP_neg_price') = (1-alpha(merged_set_1)) * sum(T$p_retail_frp_neg(merged_set_1), (sqr(p0(merged_set_1,T)-p_retail_frp(merged_set_1)))/(-2*m(merged_set_1)) ) ;
         CR_net(merged_set_1,'FRP') = CR_net(merged_set_1,'FRP_pos_price') + CR_net(merged_set_1,'FRP_neg_price');
         CR_gross(merged_set_1,'FRP_pos_price') = (1-alpha(merged_set_1)) * sum(T$(not p_retail_frp_neg(merged_set_1)), (sqr(p0(merged_set_1,T))-sqr(p_retail_frp(merged_set_1)))/(-2*m(merged_set_1)) ) ;
         CR_gross(merged_set_1,'FRP_neg_price') = (1-alpha(merged_set_1)) * sum(T$p_retail_frp_neg(merged_set_1), (sqr(p0(merged_set_1,T))-sqr(p_retail_frp(merged_set_1)))/(-2*m(merged_set_1)) ) ;
         CR_gross(merged_set_1,'FRP') = CR_gross(merged_set_1,'FRP_pos_price') + CR_gross(merged_set_1,'FRP_neg_price');

         CR_net(merged_set_1,'total') = CR_net(merged_set_1,'RTP') + CR_net(merged_set_1,'FRP') ;
         CR_gross(merged_set_1,'total') = CR_gross(merged_set_1,'RTP') + CR_gross(merged_set_1,'FRP') ;
);

* Comparison of consumer rent changes between premium on capacity and premium on energy
loop(merged_set_1$(ord(merged_set_1)>(card(merged_set_1)/2)),
         Delta_instr(merged_set_1,'CR_net','RTP') = - CR_net(merged_set_1,'RTP') + CR_net(merged_set_1-(card(merged_set_1)/2),'RTP') ;
         Delta_instr(merged_set_1,'CR_net','RTP_pos_price') = - CR_net(merged_set_1,'RTP_pos_price') + CR_net(merged_set_1-(card(merged_set_1)/2),'RTP_pos_price') ;
         Delta_instr(merged_set_1,'CR_net','RTP_neg_price') = - CR_net(merged_set_1,'RTP_neg_price') + CR_net(merged_set_1-(card(merged_set_1)/2),'RTP_neg_price') ;
         Delta_instr(merged_set_1,'CR_net','FRP') = - CR_net(merged_set_1,'FRP') + CR_net(merged_set_1-(card(merged_set_1)/2),'FRP') ;
         Delta_instr(merged_set_1,'CR_net','FRP_pos_price') = - CR_net(merged_set_1,'FRP_pos_price') + CR_net(merged_set_1-(card(merged_set_1)/2),'FRP_pos_price') ;
         Delta_instr(merged_set_1,'CR_net','FRP_neg_price') = - CR_net(merged_set_1,'FRP_neg_price') + CR_net(merged_set_1-(card(merged_set_1)/2),'FRP_neg_price') ;
         Delta_instr(merged_set_1,'CR_net','total') = - CR_net(merged_set_1,'total') + CR_net(merged_set_1-(card(merged_set_1)/2),'total') ;
         Delta_instr(merged_set_1,'CR_gross','RTP') = - CR_gross(merged_set_1,'RTP') + CR_gross(merged_set_1-(card(merged_set_1)/2),'RTP') ;
         Delta_instr(merged_set_1,'CR_gross','RTP_pos_price') = - CR_gross(merged_set_1,'RTP_pos_price') + CR_gross(merged_set_1-(card(merged_set_1)/2),'RTP_pos_price') ;
         Delta_instr(merged_set_1,'CR_gross','RTP_neg_price') = - CR_gross(merged_set_1,'RTP_neg_price') + CR_gross(merged_set_1-(card(merged_set_1)/2),'RTP_neg_price') ;
         Delta_instr(merged_set_1,'CR_gross','FRP') = - CR_gross(merged_set_1,'FRP') + CR_gross(merged_set_1-(card(merged_set_1)/2),'FRP') ;
         Delta_instr(merged_set_1,'CR_gross','FRP_pos_price') = - CR_gross(merged_set_1,'FRP_pos_price') + CR_gross(merged_set_1-(card(merged_set_1)/2),'FRP_pos_price') ;
         Delta_instr(merged_set_1,'CR_gross','FRP_neg_price') = - CR_gross(merged_set_1,'FRP_neg_price') + CR_gross(merged_set_1-(card(merged_set_1)/2),'FRP_neg_price') ;
         Delta_instr(merged_set_1,'CR_gross','total') = - CR_gross(merged_set_1,'total') + CR_gross(merged_set_1-(card(merged_set_1)/2),'total') ;

         Delta_instr(merged_set_1,'CR_net','RTP_neg_price_1') = - CR_net(merged_set_1,'RTP_neg_price_1') + CR_net(merged_set_1-(card(merged_set_1)/2),'RTP_neg_price_1') ;
         Delta_instr(merged_set_1,'CR_gross','RTP_neg_price_1') = - CR_gross(merged_set_1,'RTP_neg_price_1') + CR_gross(merged_set_1-(card(merged_set_1)/2),'RTP_neg_price_1') ;
);

$offorder

* Consumer surplus changes between different RTP shares - here only calculated for RES share 50%
loop(set_eta,
* "-1" does not matter for alpha, m and p0
* this specification works only if solution for alpha 0.01 is present
loop(subset_MPen$((eta(subset_MPen) = set_eta.val) and not subset_alpha_01(subset_MPen)),
         Delta_alpha(subset_MPen,'CR_RTP') = - alpha(subset_MPen-1)/(2*m(subset_MPen-1)) * sum(T, sqr(prtp(subset_MPen-1,T)+mp_levy(subset_MPen-1)) - sqr(prtp(subset_MPen,T)+mp_levy(subset_MPen)) )
                 - alpha(subset_MPen-1) * sum(T, qrtp(subset_MPen,T)*(prtp(subset_MPen,T)+mp_levy(subset_MPen)) - qrtp(subset_MPen-1,T)*(prtp(subset_MPen-1,T)+mp_levy(subset_MPen-1)) ) ;
         Delta_alpha(subset_MPen,'CR_FRP') = - (1-alpha(subset_MPen))/(2*m(subset_MPen)) * sum(T, sqr(pflat(subset_MPen-1)+mp_levy(subset_MPen-1)) - sqr(pflat(subset_MPen)+mp_levy(subset_MPen)) )
                 - (1-alpha(subset_MPen)) * sum(T, qfrp(subset_MPen,T)*(pflat(subset_MPen)+mp_levy(subset_MPen)) - qfrp(subset_MPen-1,T)*(pflat(subset_MPen-1)+mp_levy(subset_MPen-1)) ) ;
         Delta_alpha(subset_MPen,'CR_SWITCH') = - (alpha(subset_MPen)-alpha(subset_MPen-1))/(2*m(subset_MPen)) * sum(T, sqr(pflat(subset_MPen-1)+mp_levy(subset_MPen-1)) - sqr(prtp(subset_MPen,T)+mp_levy(subset_MPen)) )
                 - (alpha(subset_MPen)-alpha(subset_MPen-1)) * sum(T, qrtp(subset_MPen,T)*(prtp(subset_MPen,T)+mp_levy(subset_MPen)) - qfrp(subset_MPen-1,T)*(pflat(subset_MPen-1)+mp_levy(subset_MPen-1)) ) ;
         Delta_alpha(subset_MPen,'CR_total') = Delta_alpha(subset_MPen,'CR_RTP') + Delta_alpha(subset_MPen,'CR_FRP') + Delta_alpha(subset_MPen,'CR_SWITCH') ;
);
loop(subset_MPcap$((eta(subset_MPcap) = set_eta.val) and not subset_alpha_01(subset_MPcap)),
         Delta_alpha(subset_MPcap,'CR_RTP') = - alpha(subset_MPcap-1)/(2*m(subset_MPcap-1)) * sum(T, sqr(prtp(subset_MPcap-1,T)+mp_levy(subset_MPcap-1)) - sqr(prtp(subset_MPcap,T)+mp_levy(subset_MPcap)) )
                 - alpha(subset_MPcap-1) * sum(T, qrtp(subset_MPcap,T)*(prtp(subset_MPcap,T)+mp_levy(subset_MPcap)) - qrtp(subset_MPcap-1,T)*(prtp(subset_MPcap-1,T)+mp_levy(subset_MPcap-1)) ) ;
         Delta_alpha(subset_MPcap,'CR_FRP') = - (1-alpha(subset_MPcap))/(2*m(subset_MPcap)) * sum(T, sqr(pflat(subset_MPcap-1)+mp_levy(subset_MPcap-1)) - sqr(pflat(subset_MPcap)+mp_levy(subset_MPcap)) )
                 - (1-alpha(subset_MPcap)) * sum(T, qfrp(subset_MPcap,T)*(pflat(subset_MPcap)+mp_levy(subset_MPcap)) - qfrp(subset_MPcap-1,T)*(pflat(subset_MPcap-1)+mp_levy(subset_MPcap-1)) ) ;
         Delta_alpha(subset_MPcap,'CR_SWITCH') = - (alpha(subset_MPcap)-alpha(subset_MPcap-1))/(2*m(subset_MPcap)) * sum(T, sqr(pflat(subset_MPcap-1)+mp_levy(subset_MPcap-1)) - sqr(prtp(subset_MPcap,T)+mp_levy(subset_MPcap)) )
                 - (alpha(subset_MPcap)-alpha(subset_MPcap-1)) * sum(T, qrtp(subset_MPcap,T)*(prtp(subset_MPcap,T)+mp_levy(subset_MPcap)) - qfrp(subset_MPcap-1,T)*(pflat(subset_MPcap-1)+mp_levy(subset_MPcap-1)) ) ;
         Delta_alpha(subset_MPcap,'CR_total') = Delta_alpha(subset_MPcap,'CR_RTP') + Delta_alpha(subset_MPcap,'CR_FRP') + Delta_alpha(subset_MPcap,'CR_SWITCH') ;
);
);

$onorder

loop(merged_set_1,
         cost_sum(merged_set_1) = sum(I, fc(I) * OpCa(merged_set_1,I) ) + sum((I,T), (mc(I)+co2(I)*pcarb) * q(merged_set_1,I,T) ) ;
);

loop(merged_set_1$(ord(merged_set_1)>(card(merged_set_1)/2)),
         Delta_instr(merged_set_1,'capacity',I) = - OpCa(merged_set_1,I) + OpCa(merged_set_1-(card(merged_set_1)/2),I) ;
         Delta_instr(merged_set_1,'capacity','total') = sum(I, Delta_instr(merged_set_1,'capacity',I)) ;
         Delta_instr(merged_set_1,'cost','fix') = sum(I, fc(I) * (OpCa(merged_set_1-(card(merged_set_1)/2),I) - OpCa(merged_set_1,I)) ) ;
         Delta_instr(merged_set_1,'cost','var') = sum((I,T), (mc(I)+co2(I)*pcarb) * (q(merged_set_1-(card(merged_set_1)/2),I,T) - q(merged_set_1,I,T)) ) ;
         Delta_instr(merged_set_1,'cost','total') = Delta_instr(merged_set_1,'cost','fix') + Delta_instr(merged_set_1,'cost','var') ;
);

loop(merged_set_1,
         market_value(merged_set_1,I)$IR(I) = sum(T, prtp(merged_set_1,T)*q(merged_set_1,I,T) ) / sum(T, q(merged_set_1,I,T)) ;
);

loop(merged_set_1,
         euro_per_MWh(merged_set_1,'mean_price_rtp') = sum(T, prtp(merged_set_1,T)*qrtp(merged_set_1,T) ) / sum(T, qrtp(merged_set_1,T) ) ;
         euro_per_MWh(merged_set_1,'pflat') = pflat(merged_set_1) ;
         euro_per_MWh(merged_set_1,'mp_levy') = mp_levy(merged_set_1) ;
);

loop(merged_set_1,
         q_sum(merged_set_1,'q_rtp') = alpha(merged_set_1) * sum(T, qrtp(merged_set_1,T)) ;
         q_sum(merged_set_1,'q_frp') = (1-alpha(merged_set_1)) * sum(T, qfrp(merged_set_1,T)) ;
         q_sum(merged_set_1,'q_sum') = sum((I,T), q(merged_set_1,I,T)) ;
         turnover(merged_set_1,'RTP') = alpha(merged_set_1) * sum(T, (prtp(merged_set_1,T)+mp_levy(merged_set_1)) * qrtp(merged_set_1,T)) ;
         turnover(merged_set_1,'FRP') = (1-alpha(merged_set_1)) * sum(T, (pflat(merged_set_1)+mp_levy(merged_set_1)) * qfrp(merged_set_1,T)) ;
         turnover(merged_set_1,'total') = sum((I,T), q(merged_set_1,I,T) * (prtp(merged_set_1,T)+mp_levy(merged_set_1))) ;
);


execute_unload 'results.gdx'
cost_sum
q_sum
turnover
market_value
OpCa
euro_per_MWh
CR_net
CR_gross
Delta_instr
Delta_alpha
;
