$ontext
Written by Michael Pahle, Wolf-Peter Schill, Christian Gambardella and Oliver Tietjen.
We are happy to receive your feedback under michael.pahle@pik-potsdam.de.
This work is licensed under the MIT License (MIT).
For more information on the terms and conditions of this license, visit http://opensource.org/licenses/mit-license.php.
Whenever you use this code, please refer to
Michael Pahle, Wolf-Peter Schill, Christian Gambardella and Oliver Tietjen (2016):
Renewable Energy Support, Negative Prices, and Real-time Pricing
The Energy Journal, 37(SI3), 147-169, https://doi.org/10.5547/01956574.37.SI3.mpah
$offtext

*******************************************************
$eolcom #

* Choose a value for eta, e.g. -0.05:
$setglobal choose_eta "-0.05"

* Choose a value for alpha, e.g. 0.1:
$setglobal choose_alpha "0.1"

* Set a star * to switch off reading from Excel and instead use an existing GDX file:
$setglobal no_excel ""

* Set a star * to exclude offshore wind power:
$setglobal no_off "*"

* Set a star * to exclude OCGT plants:
$setglobal no_OCGT_Gas "*"

* Set one (and only one) star * to choose the support scheme:
$setglobal premium_capacity ""
$setglobal premium_energy "*"

* Set a star * to choose a demand function - iso-elastic or linear: (iso-elastic version not used in the paper)
$setglobal ISO ""
$setglobal LIN "*"

* Set a star * to solve an MPEC formulation, in which market premia are endogenously determined.
* Prior to the MPEC, an MCP will be solved in order to generate a better starting point for the MPEC problem.
* The MCP in turn requires specifying a market premium which roughly triggers the right renewable share.
$setglobal MPEC "*"

* Set a star * to use a startpoint from a previous solve (must be named "startpoint%label1%%label2%.gdx"):
$setglobal startpoint ""

* Set the share of renewables in overall power supply that has to be reached:
scalar RES_share /0.5/ ;

*******************************************************
* Automatic definition of switches and sanity checks - do not modify!
$setglobal with_off ""
%no_off%$setglobal with_off "*"
$setglobal with_OCGT_Gas ""
%no_OCGT_Gas%$setglobal with_OCGT_Gas "*"

$if "%ISO%" == "%LIN%" $abort Choose either ISO or LIN!
$setglobal check "%premium_capacity%%premium_energy%"
$if NOT "%check%" == "*" $abort Please choose support instrument!

$setglobal label1 ""
$setglobal label2 ""
$if "%premium_capacity%" == "*" $setglobal label1 "_MPcap"
$if "%premium_energy%" == "*" $setglobal label1 "_MPen"
$if "%ISO%" == "*" $setglobal label2 "_ISO"
$if "%LIN%" == "*" $setglobal label2 "_LIN"
*******************************************************

* Decrease size of list file:
options
savepoint = 1 ,
solprint = off ,
limrow = 0 ,
limcol = 0 ;

Sets
%no_off%%no_OCGT_Gas%I           Technology segment      /Wind_on,Wind_off,PV,Coal,CCGT,OCGT_Gas,OCGT_Oil/
%with_off%%no_OCGT_Gas%I         Technology segment      /Wind_on,PV,Coal,CCGT,OCGT_Gas,OCGT_Oil/
%no_off%%with_OCGT_Gas%I         Technology segment      /Wind_on,Wind_off,PV,Coal,CCGT,OCGT_Oil/
%with_off%%with_OCGT_Gas%I       Technology segment      /Wind_on,PV,Coal,CCGT,OCGT_Oil/

%no_OCGT_Gas%ID(I)               Dispatchable technology segment         /Coal,CCGT,OCGT_Gas,OCGT_Oil/
%with_OCGT_Gas%ID(I)             Dispatchable technology segment         /Coal,CCGT,OCGT_Oil/

%no_off%IR(I)                    RES                                     /Wind_on,Wind_off,PV/
%with_off%IR(I)                  RES                                     /Wind_on,PV/

T                                Hour t in given year                    /1*8760/
;

Alias (T,TT), (I,II) ;

Scalars
alpha         Share of consumers on real-time pricing            /%choose_alpha%/
eta           Price elasticity                                   /%choose_eta%/
* In the EJ application, we asume a carbon price of zero because of renewable support schemes and a RES target
pcarb         Carbon price                                       /0/
;


Parameters
mc(I)         Variable costs technology i
co2(I)        Carbon emission efficiency factor
fc(I)         Fixed Cost Annuity technology I (per MW)
a(T)          Demand anchor point (computed from input data and elasticity)
MP_par_cap(I) Market premium on capacity (parameter)
MP_par_en     Market premium on energy (parameter)
av_(T,I)      Technology specific availability factor (T I)
av(I,T)       Technology specific availability factor (I T)
p_data(T)     Market price input data
q_data(T)     Consumption input data
p0(T)         Linear demand interception
m             Linear demand slope
;

* Read in parameters from Excel:
$onecho >temp.tmp
par=mc          rng=mc!a1:b9        rdim=1 cdim=0
par=co2         rng=co2!a1:b9       rdim=1 cdim=0
par=fc          rng=fc!a1:b9        rdim=1 cdim=0
par=av_         rng=av!a1:d8761     rdim=1 cdim=1
par=p_data      rng=p!a1:b8760      rdim=1 cdim=0
par=q_data      rng=q!a1:b8760      rdim=1 cdim=0
$offecho

%no_excel%$call "gdxxrw input_data.xlsx @temp.tmp o=input_data";
$GDXin input_data.gdx
$load mc co2 fc av_ p_data q_data
* change (T,I) to (I,T)
av(I,T) = av_(T,I) ;
av(I,T)$(ID(I)) = 1 ;
;

* Compute demand function parameters:
* Iso-elastic demand: we use weighted average prices to derive a(T) assuming alpha=0
a(T) = q_data(T)/( (sum(TT, p_data(TT)*q_data(TT))/sum(TT, q_data(TT))) **eta ) ;

* Calculate m for linear demand function: price = p0 + m*quantity (inverse demand function)
* m = price/(eta*quantity) and use weighted average prices and quantities
m = sum(T, p_data(T)*q_data(T))/sum(T, q_data(T)) / (eta * (sum(T, q_data(T)*p_data(T))/sum(T, p_data(T))) ) ;
p0(T) = p_data(T) - m*q_data(T) ;


Variables
prtp(T)          Electricity Market Equilibrium Price in hour T
pflat            Electricity flat rate tarif
mp_levy          Marekt premium levy to be paid on top of electricity price
obj              Objective value upper level MPEC
;


Positive Variables
q(I,T)           Equilibrium Quantity
OpCa(I)          Optimal Capacity Technology I Contingent on Investment Cost Annuities
lambda1(I,T)     Shadow price of capacity constraint from plant i of technology type j in hour t
MP_var_cap(I)    Market premium capacity (variable)
MP_var_en        Market premium energy (variable)
;


Equations
MPEC_upper_MP_ISO        Upper level function of MPEC for iso-elastic demand function
MPEC_upper_MP_LIN        Upper level function of MPEC for linear demand function
RES_constr_equality      Renewable constraint with =e=

RTmkt_iso                Retail market (isoleastic demand)
mkt_iso                  Electricity Market Equilibrium Condition MCP (isoleastic demand)
RTmkt_lin                Retail market (linear demand)
mkt_lin                  Electricity Market Equilibrium Condition MCP (linear demand)

profit_MP_cap            Zero Profit Condition with respect to generation variables for MCP and MPEC with premium on capacity
profit_MP_en             Zero Profit Condition with respect to generation variables for MPEC with premium on energy
focCap_MP_par_cap        F.O.C. Optimal Capacity (MP_cap as a parameter)
focCap_MP_var_cap        F.O.C. Optimal Capacity (MP_cap as a variable)
focCap                   F.O.C. Optimal Capacity (for premium on energy)

capconstr                Reserve Margin Constraint (i.e. capacity constraint)

levy_MP_par_cap          Compute levy (MP_cap as a parameter)
levy_MP_var_cap          Compute levy (MP_cap as a variable)
levy_MP_par_en           Compute levy (MP_en as a parameter)
levy_MP_var_en           Compute levy (MP_en as a variable)
;


* iso-elastic version not used in the paper; place holder objective function 
MPEC_upper_MP_ISO..     obj =e= 1 ;

MPEC_upper_MP_LIN..     obj =e= ( sum(T, alpha*(sqr(prtp(T)+mp_levy)-sqr(p0(T)))/(2*m)   # Gross surplus of rtp consumers
                         + (1-alpha)*(sqr(pflat+mp_levy)-sqr(p0(T)))/(2*m) )             # Gross surplus of flat consumers
                         - sum((I,T), (mc(I)+co2(I)*pcarb)*q(I,T))                       # variable costs
                         - sum(I,fc(I)*card(T)/8760*OpCa(I) )                            # fixed costs
                         ) /1e9 ;                                                        # scaling factor

RES_constr_equality..   sum((I,T)$(IR(I)), q(I,T)) =e= RES_share * sum((I,T), q(I,T)) ;

mkt_iso(T)..            sum(I,q(I,T)) =e= alpha*a(T)*(prtp(T)+mp_levy)**eta + (1-alpha)*a(T)*(pflat+mp_levy)**eta;
RTmkt_iso..             sum(T,(pflat-prtp(T))*(1-alpha)*a(T)*(pflat+mp_levy)**eta) =e= 0;
mkt_lin(T)..            sum(I,q(I,T)) =g= alpha*(prtp(T)+mp_levy-p0(T))/m + (1-alpha)*(pflat+mp_levy-p0(T))/m ;
RTmkt_lin..             sum(T,(pflat-prtp(T))*(1-alpha)*(pflat+mp_levy-p0(t))/m) =e= 0;

profit_MP_cap(I,T)..    mc(I)+co2(I)*pcarb-MP_par_en$(IR(I))+lambda1(I,T) =g= prtp(T);
profit_MP_en(I,T)..     mc(I)+co2(I)*pcarb-MP_var_en$(IR(I))+lambda1(I,T) =g= prtp(T);
focCap_MP_par_cap(I)..  fc(I)*card(T)/8760-MP_par_cap(I)$(IR(I)) =g= sum(T,lambda1(I,T)*av(I,T)) ;
focCap_MP_var_cap(I)..  fc(I)*card(T)/8760-MP_var_cap(I)$(IR(I)) =g= sum(T,lambda1(I,T)*av(I,T)) ;
focCap(I)..             fc(I)*card(T)/8760 =g= sum(T,lambda1(I,T)*av(I,T)) ;

capconstr(I,T)..        OpCa(I)*av(I,T) =g= q(I,T);

levy_MP_par_cap..       mp_levy =e= sum(I, OpCa(I)*MP_par_cap(I)$(IR(I)))/(sum((I,T), q(I,T)));
levy_MP_var_cap..       mp_levy =e= sum(I, OpCa(I)*MP_var_cap(I)$(IR(I)))/(sum((I,T), q(I,T)));
levy_MP_par_en..        mp_levy =e= sum((I,T)$(IR(I)), q(I,T)*MP_par_en)/(sum((I,T), q(I,T)));
levy_MP_var_en..        mp_levy =e= sum((I,T)$(IR(I)), q(I,T)*MP_var_en)/(sum((I,T), q(I,T)));


model INST_MCP_ISO /
mkt_iso.prtp
Rtmkt_iso.pflat
profit_MP_cap.q
focCap_MP_par_cap.OpCa
capconstr.lambda1
%premium_energy%levy_MP_par_cap.mp_levy
%premium_capacity%levy_MP_par_en.mp_levy
/
;

model INST_MCP_LIN /
mkt_lin.prtp
Rtmkt_lin.pflat
profit_MP_cap.q
focCap_MP_par_cap.OpCa
capconstr.lambda1
%premium_energy%levy_MP_par_cap.mp_levy
%premium_capacity%levy_MP_par_en.mp_levy
/
;

model INST_MPEC_ISO /
%premium_capacity%$ontext
MPEC_upper_MP_ISO
profit_MP_cap.q
focCap_MP_var_cap.OpCa
levy_MP_var_cap.mp_levy
$ontext
$offtext
%premium_energy%$ontext
MPEC_upper_MP_ISO
profit_MP_en.q
focCap.OpCa
levy_MP_var_en.mp_levy
$ontext
$offtext
RES_constr_equality
mkt_iso.prtp
Rtmkt_iso.pflat
capconstr.lambda1
/
;

model INST_MPEC_LIN /
%premium_capacity%$ontext
MPEC_upper_MP_LIN
profit_MP_cap.q
focCap_MP_var_cap.OpCa
levy_MP_var_cap.mp_levy
$ontext
$offtext
%premium_energy%$ontext
MPEC_upper_MP_LIN
profit_MP_en.q
focCap.OpCa
levy_MP_var_en.mp_levy
$ontext
$offtext
RES_constr_equality
mkt_lin.prtp
Rtmkt_lin.pflat
capconstr.lambda1
/
;


* Specify market premia. This is only relevant for MCP in case no starting point from a previous solve is used:
%premium_capacity%$ontext
%ISO%MP_par_cap('Wind_on') = 93636/card(T)*8760 ;        # Possible starting point for linear demand function, eta 0.05, alpha 0.1
%ISO%MP_par_cap('PV') = 53664/card(T)*8760 ;             # Possible starting point for linear demand function, eta 0.05, alpha 0.1
*%ISO%MP_par_cap(IR) = 0.7*fc(IR)*card(T)/8760 ;         # Alternative approach for linear demand function: relate market premia to costs
%LIN%MP_par_cap('Wind_on') = 92275/card(T)*8760 ;        # Possible starting point for iso-elastic demand function, eta 0.05, alpha 0.1
%LIN%MP_par_cap('PV') = 52510/card(T)*8760 ;             # Possible starting point for iso-elastic demand function, eta 0.05, alpha 0.1
*%LIN%MP_par_cap(IR) = 0.7*fc(IR)*card(T)/8760 ;         # Alternative approach for iso-elastic demand function: relate market premia to costs
MP_par_en = 0 ;
$ontext
$offtext

%premium_energy%$ontext
%ISO%MP_par_en = 86 ;                                    # Possible starting point for linear demand function, eta 0.05, alpha 0.1
%LIN%MP_par_en = 66 ;                                    # Possible starting point for iso-elastic demand function, eta 0.05, alpha 0.1
MP_par_cap(I) = 0 ;
$ontext
$offtext


* Enable scaling
INST_MCP_ISO.scaleopt = 1;
INST_MCP_LIN.scaleopt = 1;
INST_MPEC_ISO.scaleopt = 1;
INST_MPEC_LIN.scaleopt = 1;

option reslim=86400;

* Reporting:
set versions /ISO,LIN/ ;
parameter report_parameters, report_parameters_T, report, report_I, report_I_T, report_T ;


**** SOLVE MCP ISO: ************************************************************

%ISO%$goto skip_MCP_ISO
* Starting values:
* prtp: set to p_data, but use 0.0001 in case input data <= 0 in order to avoid errors relate to initial values in equation mkt_iso(T)
prtp.l(T) = p_data(T)$(p_data(T) > 0) + 0.0001$(p_data(T) <= 0) ;
pflat.l = 100 ;
q.l(I,T) = 10000 ;
OpCa.l(I) = 10000 ;
mp_levy.l = sum(IR, OpCa.l(IR)*MP_par_cap(IR))/(sum((I,T), q.l(I,T))) ;
lambda1.l(I,T) = prtp.l(T) - mc(I)-co2(I)*pcarb+MP_par_en$(IR(I)) ;

* Scaling (may be improved):
RTmkt_iso.scale = 1e+3 ;
levy_MP_par_cap.scale = 1e+3 ;
mkt_iso.scale(T) = 1e+6 ;

%startpoint%$ontext
MP_var_cap.l(IR) = 0 ;   # Initialize MP_var_cap
MP_var_en.l = 0 ;        # Initialize MP_var_en
* If there is already a solution of a related model run, use it:
execute_loadpoint 'startpoint%label1%%label2%.gdx' ;
* Avoid errors because of negative initial prices:
prtp.l(T)$((prtp.l(T)+mp_levy.l) <= 0) = -mp_levy.l + 0.1 ;
MP_par_en = MP_var_en.l ;
MP_par_cap(I) = MP_var_cap.l(I) ;
$ontext
$offtext

SOLVE INST_MCP_ISO USING MCP;

$label skip_MCP_ISO


**** SOLVE MPEC ISO: ***********************************************************

* Solve MPEC only after MCP_ISO:
%ISO%$goto skip_MPEC_ISO
%MPEC%$goto skip_MPEC_ISO
MP_var_cap.l(I)$(IR(I)) = MP_par_cap(I) ;
MP_var_en.l = MP_par_en ;

* Avoid pre-solve errors because of negative prices:
prtp.l(T)$((prtp.l(T)+mp_levy.l) <= 0) = -mp_levy.l + 0.1 ;
prtp.lo(T) = -mp_levy.l + 0.1 ;

SOLVE INST_MPEC_ISO maximizing obj using MPEC;
%premium_energy%SOLVE INST_MPEC_ISO maximizing obj using MPEC;

$label skip_MPEC_ISO

* Report ISO solution:
%ISO%$ontext
report('alpha') = alpha ;
report('eta') = eta ;
report_T('a',T) = a(T) ;
report('RES share') = sum((IR,T), q.l(IR,T)) / sum((I,T), q.l(I,T)) ;
report('Total gen') = sum((I,T), q.l(I,T)) ;
report_I('Capacities',I) = OpCa.l(I) ;
report_I('Capacity shares',I) = OpCa.l(I) / sum(II, OpCa.l(II)) ;
report_I('Generation',I) = sum(T, q.l(I,T)) ;
report_I('Generation shares',I) = sum(T, q.l(I,T)) / report('Total gen') ;
report_I_T('Generation',I,T) = q.l(I,T) + 0.0000000001$(not q.l(I,T)) ;
report_I('Producer Rent',I) = sum(T, (prtp.l(T)-mc(I)-co2(I)*pcarb+MP_var_en.l$(IR(I)))*q.l(I,T)) - (fc(I)*card(T)/8760-MP_var_cap.l(I)$(IR(I)))*OpCa.l(I) ;
*report('Welfare') = # tbd # ;
report('pflat') = pflat.l ;
report_T('prtp',T) = prtp.l(T) + 0.0000000001$(not prtp.l(T)) ;
report('mp_levy') = mp_levy.l ;
%premium_energy%report_I('MP_var_cap',I) = MP_var_cap.l(I) ;
%premium_capacity%report('MP_var_en') = MP_var_en.l ;
$ontext
$offtext


**** SOLVE MCP LIN: ************************************************************

%LIN%$goto skip_MCP_LIN
* Starting values:
prtp.l(T) = p_data(T) ;
pflat.l = 100 ;
q.l(I,T) = 100000 ;
OpCa.l(I) = 100000 ;
mp_levy.l = sum(IR, OpCa.l(IR)*MP_par_cap(IR))/(sum((I,T), q.l(I,T))) ;
lambda1.l(I,T) = prtp.l(T) - mc(I)+MP_par_en$(IR(I)) ;

* Scaling (may be improved):
RTmkt_lin.scale = 1e+6 ;
levy_MP_par_cap.scale = 1e+3 ;
*mkt_lin.scale(T) = 1e+6 ;

%startpoint%$ontext
* If there is already a solution of a related model run, use it:
MP_var_cap.l(IR) = 0 ;   # Initialize MP_var_cap
MP_var_en.l = 0 ;        # Initialize MP_var_en
execute_loadpoint 'startpoint%label1%%label2%.gdx' ;
MP_par_cap(IR) = MP_var_cap.l(IR) ;
MP_par_en = MP_var_en.l ;
$ontext
$offtext

SOLVE INST_MCP_LIN USING MCP;

$label skip_MCP_LIN


**** SOLVE MPEC LIN: ***********************************************************

* Solve MPEC only after MCP_LIN:
%LIN%$goto skip_MPEC_LIN
%MPEC%$goto skip_MPEC_LIN
MP_var_cap.l(I)$(IR(I)) = MP_par_cap(I) ;
MP_var_en.l = MP_par_en ;

SOLVE INST_MPEC_LIN maximizing obj using MPEC;

$label skip_MPEC_LIN

* Report LIN solution:
%LIN%$ontext
report('Model status') = INST_MPEC_LIN.modelstat ;
report('alpha') = alpha ;
report('eta') = eta ;
report('m') = m ;
report_T('p0',T) = p0(T) ;
report('RES share') = sum((IR,T), q.l(IR,T)) / sum((I,T), q.l(I,T)) ;
report('Total gen') = sum((I,T), q.l(I,T)) ;
report_I('Capacities',I) = OpCa.l(I) ;
report_I('Capacity shares',I) = OpCa.l(I) / sum(II, OpCa.l(II)) ;
report_I('Generation',I) = sum(T, q.l(I,T)) ;
report_I('Generation shares',I) = sum(T, q.l(I,T)) / report('Total gen') ;
report_I_T('Generation',I,T) = q.l(I,T) + 0.0000000001$(not q.l(I,T)) ;
report_I('Producer Rent',I) = sum(T, (prtp.l(T)-mc(I)-co2(I)*pcarb+MP_var_en.l$(IR(I)))*q.l(I,T)) - (fc(I)*card(T)/8760-MP_var_cap.l(I)$(IR(I)))*OpCa.l(I) ;
report('Welfare') = sum(T, alpha*(sqr(prtp.l(T)+mp_levy.l)-sqr(p0(T)))/(2*m) + (1-alpha)*(sqr(pflat.l+mp_levy.l)-sqr(p0(T)))/(2*m) ) - sum((I,T), (mc(I)+co2(I)*pcarb)*q.l(I,T)) - sum(I,fc(I)*card(T)/8760*OpCa.l(I) ) ;
report('pflat') = pflat.l ;
report_T('prtp',T) = prtp.l(T) + 0.0000000001$(not prtp.l(T)) ;
report('mp_levy') = mp_levy.l ;
%premium_energy%report_I('MP_var_cap',I) = MP_var_cap.l(I) ;
%premium_capacity%report('MP_var_en') = MP_var_en.l ;
$ontext
$offtext

execute_unload "eta_%choose_eta%alpha_%choose_alpha%%label1%%label2%.gdx",report,report_I,report_I_T,report_T;
